/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * Christopher Blizzard.
 * Portions created by the Initial Developer are Copyright (C) 2001
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Christopher Blizzard <blizzard@mozilla.org>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "nsCOMPtr.h"
#include "nsIDOMMouseEvent.h"

#include "nsIDOMKeyEvent.h"
#include "nsIDOMUIEvent.h"

#include "EmbedEventListener.h"
#include "EmbedPrivate.h"

EmbedEventListener::EmbedEventListener(void)
{
  mOwner = nsnull;
}

EmbedEventListener::~EmbedEventListener()
{
}

NS_IMPL_ISUPPORTS1(EmbedEventListener, nsIDOMEventListener)

nsresult
EmbedEventListener::Init(EmbedPrivate *aOwner)
{
  mOwner = aOwner;
  return NS_OK;
}

NS_IMETHODIMP
EmbedEventListener::KeyDown(nsIDOMEvent* aDOMEvent)
{
  // Return FALSE to this function to mark the event as not
  // consumed...
  gint return_val = FALSE;
  g_signal_emit(G_OBJECT(mOwner->mOwningWidget),
                moz_embed_signals[DOM_KEY_DOWN], 0,
                (void *)aDOMEvent, &return_val);
  if (return_val) {
    aDOMEvent->StopPropagation();
    aDOMEvent->PreventDefault();
  }
  return NS_OK;
}

NS_IMETHODIMP
EmbedEventListener::KeyUp(nsIDOMEvent* aDOMEvent)
{
  // return FALSE to this function to mark this event as not
  // consumed...
  gint return_val = FALSE;
  g_signal_emit(G_OBJECT(mOwner->mOwningWidget),
                moz_embed_signals[DOM_KEY_UP], 0,
                (void *)aDOMEvent, &return_val);
  if (return_val) {
    aDOMEvent->StopPropagation();
    aDOMEvent->PreventDefault();
  }
  return NS_OK;
}

NS_IMETHODIMP
EmbedEventListener::KeyPress(nsIDOMEvent* aDOMEvent)
{
  // Return TRUE from your signal handler to mark the event as consumed.
  gint return_val = FALSE;
  g_signal_emit(G_OBJECT(mOwner->mOwningWidget),
                moz_embed_signals[DOM_KEY_PRESS], 0,
                (void *)aDOMEvent, &return_val);
  if (return_val) {
    aDOMEvent->StopPropagation();
    aDOMEvent->PreventDefault();
  }
  return NS_OK;
}

NS_IMETHODIMP
EmbedEventListener::MouseDown(nsIDOMEvent* aDOMEvent)
{
  // Return TRUE from your signal handler to mark the event as consumed.
  gint return_val = FALSE;
  g_signal_emit(G_OBJECT(mOwner->mOwningWidget),
                moz_embed_signals[DOM_MOUSE_DOWN], 0,
                (void *)aDOMEvent, &return_val);
  if (return_val) {
    aDOMEvent->StopPropagation();
    aDOMEvent->PreventDefault();
  }
  return NS_OK;
}

NS_IMETHODIMP
EmbedEventListener::MouseUp(nsIDOMEvent* aDOMEvent)
{
  // Return TRUE from your signal handler to mark the event as consumed.
  gint return_val = FALSE;
  g_signal_emit(G_OBJECT(mOwner->mOwningWidget),
                moz_embed_signals[DOM_MOUSE_UP], 0,
                (void *)aDOMEvent, &return_val);
  if (return_val) {
    aDOMEvent->StopPropagation();
    aDOMEvent->PreventDefault();
  }
  return NS_OK;
}

NS_IMETHODIMP
EmbedEventListener::MouseClick(nsIDOMEvent* aDOMEvent)
{
  // Return TRUE from your signal handler to mark the event as consumed.
  gint return_val = FALSE;
  g_signal_emit(G_OBJECT(mOwner->mOwningWidget),
                moz_embed_signals[DOM_MOUSE_CLICK], 0,
                (void *)aDOMEvent, &return_val);
  if (return_val) {
    aDOMEvent->StopPropagation();
    aDOMEvent->PreventDefault();
  }
  return NS_OK;
}

NS_IMETHODIMP
EmbedEventListener::MouseDblClick(nsIDOMEvent* aDOMEvent)
{
  // Return TRUE from your signal handler to mark the event as consumed.
  gint return_val = FALSE;
  g_signal_emit(G_OBJECT(mOwner->mOwningWidget),
                moz_embed_signals[DOM_MOUSE_DBL_CLICK], 0,
                (void *)aDOMEvent, &return_val);
  if (return_val) {
    aDOMEvent->StopPropagation();
    aDOMEvent->PreventDefault();
  }
  return NS_OK;
}

NS_IMETHODIMP
EmbedEventListener::MouseOver(nsIDOMEvent* aDOMEvent)
{
  // Return TRUE from your signal handler to mark the event as consumed.
  gint return_val = FALSE;
  g_signal_emit(G_OBJECT(mOwner->mOwningWidget),
                moz_embed_signals[DOM_MOUSE_OVER], 0,
                (void *)aDOMEvent, &return_val);
  if (return_val) {
    aDOMEvent->StopPropagation();
    aDOMEvent->PreventDefault();
  }
  return NS_OK;
}

NS_IMETHODIMP
EmbedEventListener::MouseOut(nsIDOMEvent* aDOMEvent)
{
  // Return TRUE from your signal handler to mark the event as consumed.
  gint return_val = FALSE;
  g_signal_emit(G_OBJECT(mOwner->mOwningWidget),
                moz_embed_signals[DOM_MOUSE_OUT], 0,
                (void *)aDOMEvent, &return_val);
  if (return_val) {
    aDOMEvent->StopPropagation();
    aDOMEvent->PreventDefault();
  }
  return NS_OK;
}

NS_IMETHODIMP
EmbedEventListener::Activate(nsIDOMEvent* aDOMEvent)
{
  // Return TRUE from your signal handler to mark the event as consumed.
  gint return_val = FALSE;
  g_signal_emit(G_OBJECT(mOwner->mOwningWidget),
                moz_embed_signals[DOM_ACTIVATE], 0,
                (void *)aDOMEvent, &return_val);
  if (return_val) {
    aDOMEvent->StopPropagation();
    aDOMEvent->PreventDefault();
  }
  return NS_OK;
}

NS_IMETHODIMP
EmbedEventListener::FocusIn(nsIDOMEvent* aDOMEvent)
{
  // Return TRUE from your signal handler to mark the event as consumed.
  gint return_val = FALSE;
  g_signal_emit(G_OBJECT(mOwner->mOwningWidget),
                moz_embed_signals[DOM_FOCUS_IN], 0,
                (void *)aDOMEvent, &return_val);
  if (return_val) {
    aDOMEvent->StopPropagation();
    aDOMEvent->PreventDefault();
  }
  return NS_OK;
}

NS_IMETHODIMP
EmbedEventListener::FocusOut(nsIDOMEvent* aDOMEvent)
{
  // Return TRUE from your signal handler to mark the event as consumed.
  gint return_val = FALSE;
  g_signal_emit(G_OBJECT(mOwner->mOwningWidget),
                moz_embed_signals[DOM_FOCUS_OUT], 0,
                (void *)aDOMEvent, &return_val);
  if (return_val) {
    aDOMEvent->StopPropagation();
    aDOMEvent->PreventDefault();
  }
  return NS_OK;
}

NS_IMETHODIMP
EmbedEventListener::HandleEvent(nsIDOMEvent* aDOMEvent)
{
  nsAutoString eventType;
  aDOMEvent->GetType(eventType);

  if (eventType.EqualsLiteral("keydown"))
    return KeyDown(aDOMEvent);
  else if (eventType.EqualsLiteral("keyup"))
    return KeyUp(aDOMEvent);
  else if (eventType.EqualsLiteral("keypress"))
    return KeyPress(aDOMEvent);
  else if (eventType.EqualsLiteral("mousedown"))
    return MouseDown(aDOMEvent);
  else if (eventType.EqualsLiteral("mouseup"))
    return MouseUp(aDOMEvent);
  else if (eventType.EqualsLiteral("click"))
    return MouseClick(aDOMEvent);
  else if (eventType.EqualsLiteral("dblclick"))
    return MouseDblClick(aDOMEvent);
  else if (eventType.EqualsLiteral("mouseover"))
    return MouseOver(aDOMEvent);
  else if (eventType.EqualsLiteral("mouseout"))
    return MouseOut(aDOMEvent);
  else if (eventType.EqualsLiteral("DOMActivate"))
    return Activate(aDOMEvent);
  else if (eventType.EqualsLiteral("DOMFocusIn"))
    return FocusIn(aDOMEvent);
  else if (eventType.EqualsLiteral("DOMFocusOut"))
    return FocusOut(aDOMEvent);
  
  NS_ERROR("Unexpected event type");
  return NS_OK;
}
